.logo_djeym {
    margin: 0 0 10px;
}

.logo_djeym img {
    width: 160px;
}

#id_center_map,
#id_check_icon_offset_map {
    width: 80%;
    height: 400px;
    position: relative;
}

label[for="id_center_map"],
label[for="id_check_icon_offset"] {
    font-weight: bold;
    color: #333;
    float: none !important;
    width: auto !important;
}

label[for="id_disable_standard_icons"] {
    color: #79aec8;
    font-weight: bold;
    font-size: 14px;
}

.editing_map_link {
    margin: 0 0 20px;
}

.editing_map_link a {
    position: relative;
    text-decoration: none !important;
    color: #000 !important;
}

.editing_map_link__a__wrapper {
    display: inline-block;
    background-color: #ffdb4d;
    border: 5px solid #ffdb4d;
    padding: 0 3px 1px 7px;
}

.editing_map_link__a__wrapper img {
    height: 16px;
    position: relative;
    top: 3px;
    right: 5px;
}

#id_editing_map_link__harmonic_1,
#id_editing_map_link__harmonic_2 {
    display: inline-block;
    width: 5px;
    height: 31px;
    position: relative;
    top: 10px;
    left: 0;
    background-color: #ffdb4d;
}

#id_triangle-right {
    display: inline-block;
    width: 0;
    height: 0;
    position: relative;
    top: 11px;
    left: 0;
    border-top: 16px solid transparent;
    border-left: 12px solid #ffdb4d;
    border-bottom: 16px solid transparent;
}

.hint_center_map,
.hint_check_icon_offset {
    color: #1e98ff;
    font-size: 16px;
    font-weight: bold;
    margin: 0 0 10px;
    font-style: italic;
}

.field-check_icon_offset,
.hint_check_icon_offset {
    display: none;
}

.djeym_hint_custom_icons {
    margin: 10px 0 20px;
    color: #1e98ff;
    font-size: 16px;
    font-weight: bold;
    font-style: italic;
}

.hint_offset {
    height: 32px;
    position: relative;
    top: -1px;
    left: 5px;
}

.view_icon {
    font-size: 28px;
    position: relative;
    top: 7px;
    left: 10px;
    color: #666;
}

.view_icon {
    position: relative;
    left: 10px;
}

.category_icon {
    color: #666;
    font-size: 20px;
}

label[for="id_is_cluster"] {
    color: #79aec8;
    font-weight: bold;
    font-size: 14px;
}

#id_custom_cluster_icon,
#id_collection_example_icon,
#id_tile_screenshot,
#id_icon_load_indicator {
    display: none;
    max-height: 60px;
    position: relative;
    top: 0;
    left: 20px;
}

.custom_cluster_icon_wrapper,
.icon_collection_wrapper,
.tile_screenshot_wrapper,
.load_indicator_icon_wrapper {
    overflow: visible !important;
}

.djeym_category_color {
    width: 26px;
    height: 26px;
}

/* Marker offset - Buttons for touch screen. */

.marker_offset_btn {
    display: inline-block;
    margin-left: 10px;
    cursor: pointer;
}

/* Import/Export - Icon Collection.
   Import/Export - Source tile layer */

.import_icon_collection_link,
.import_tile_source_link {
    background-color: #ffdb4d !important;
    color: #000 !important;
    background-image: url(../img/plus.svg) !important;
}

.export_tile_source_link {
    background-color: #ffdb4d !important;
    color: #000 !important;
    background-image: url(../img/import.svg) !important;
}

.export_icon_collection_link {
    position: relative;
    background-color: #ffdb4d;
    color: #000 !important;
    border-radius: 20px;
    padding: 5px 14px 5px;
    text-transform: uppercase;
    font-weight: 400;
    letter-spacing: 0.5px;
    font-size: 11px;
}

.export_icon_collection_link div {
    display: inline-block;
    width: 16px;
    height: 16px;
    position: relative;
    top: 4px;
    left: 3px;
    background: url(../img/import.svg) no-repeat center;
    background-size: cover;
}

.tile_go_to_website_btn {
    display: inline-block;
    margin-left: 10px;
}

/* Modal lock */

#djeymModalLock {
    display: none;
    position: absolute;
    top: 0;
    right: 0;
    bottom: 0;
    left: 0;
    z-index: 10000;
    background-color: #fff;
    opacity: .85;
    transition-property: opacity;
    transition-duration: 1s;
    transition-delay: 0s;
    transition-timing-function: linear;
}

.djeym-load-indicator {
    width: 128px;
    height: 128px;
    position: absolute;
    top: 50%;
    left: 50%;
    margin: -64px 0 0 -64px;
    content: url('../../djeym/img/spinner.svg');
    animation: djeym-spin 1s infinite linear;
}

@keyframes djeym-spin {
    0% {
        transform: rotate(0deg);
    }
    100% {
        transform: rotate(359deg);
    }
}

/* Helpers */

.hidden {
    display: none !important;
}